//
//  main.swift
//  Connect4_CLI
//
//  Created by Adam BONAFOS on 13/01/2025.
//

import Foundation
import Model

print("🎮 Bienvenue dans Connect4 / Tic-Tac-Toe !")

/// Fonction de lecture utilisateur
func read(question: String) -> String {
    print(question)
    let answer = readLine()
    return answer ?? ""
}

/// Affichage du plateau de jeu
public func display(board: Board) {
    var boardString = ""
    for row in (0..<board.rowNb).reversed() {
        for column in 0..<board.columnNb {
            let token = board[row, column]
            let emoji: String
            switch token {
            case .red:
                emoji = "🔴"  // Rond rouge
            case .yellow:
                emoji = "🟡"  // Rond jaune
            case .empty:
                emoji = "⚪"  // Rond gris (vide)
            }
            boardString += emoji + " "
        }
        boardString += "\n"
    }
    print(boardString)
}

/// Demande et valide une entrée utilisateur
func getUserInput(question: String, validValues: [Int]) -> Int {
    var userInput: Int?
    repeat {
        let input = read(question: question)
        if let value = Int(input), validValues.contains(value) {
            userInput = value
        } else {
            print("❌ Entrée invalide. Veuillez entrer une valeur parmi : \(validValues)")
        }
    } while userInput == nil
    return userInput!
}

/// Sélectionne le type de joueur
func selectPlayer(playerNumber: Int, rules: Rules, color: Token) -> Player {
    let choice = getUserInput(question: "Joueur \(playerNumber) : 1️⃣ Humain | 2️⃣ IA ?", validValues: [1, 2])
    return choice == 1
        ? HumanPlayer(rules: rules, color: color, name: "Joueur \(playerNumber)", CLI: read)
        : RandomPlayer(rules: rules, color: color, name: "BOT \(playerNumber)")
}

/// Sélection du jeu et création de la partie
func selectGame() -> Game {
    let gameChoice = getUserInput(question: "Choisissez le jeu : 1️⃣ Puissance 4 | 2️⃣ Tic-Tac-Toe ?", validValues: [1, 2])

    if gameChoice == 1 {
        print("🎮 Démarrage de **Puissance 4** !")
        let rules = Connect4Rules(piecesToAlign: 4)
        let player1 = selectPlayer(playerNumber: 1, rules: rules, color: .red)
        let player2 = selectPlayer(playerNumber: 2, rules: rules, color: .yellow)
        return Game(rules: rules, players: [player1, player2])
    } else {
        print("🎮 Démarrage de **Tic-Tac-Toe** !")
        let rules = TicTacToeRules(piecesToAlign: 3)
        let player1 = selectPlayer(playerNumber: 1, rules: rules, color: .red)
        let player2 = selectPlayer(playerNumber: 2, rules: rules, color: .yellow)
        return Game(board: Board(rowsNb: 3, columnsNb: 3), rules: rules, players: [player1, player2])
    }
}

// 🔹 Initialisation du jeu
var game = selectGame()

// 🔹 Ajout des commandes d'affichage
game.on(
    event: .turn,
    callback: { name in
        if let name { print("🌀 Au tour de : \(name)") }
    }
)
game.on(
    event: .badMove,
    callback: { _ in print("❌ Move invalide, choisissez-en un autre.")
    })

game.on(
    event: .victory,
    callback: { name in
        if let name { print("🏆 Victoire de : \(name) !") }
    })

// 🔹 Lancement du jeu
game.start(display: display)
