import XCTest
@testable import Model

final class IsGameOverTest: XCTestCase {
    func testHorizontal() {
        let rules = Connect4Rules(piecesToAlign: 4)
        var board = Board()
        board[0, 0] = Token.red
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 6] = Token.yellow
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 1] = Token.red
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 5] = Token.yellow
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 2] = Token.red
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 4] = Token.yellow
        XCTAssertFalse(rules.isGameOver(board: board).result)
        board[0, 3] = Token.red
        XCTAssertTrue(rules.isGameOver(board: board).result)
        XCTAssertEqual(rules.isGameOver(board: board).winner, Token.red)
    }
    func testFullGridWithoutWin() {
        let rules = TicTacToeRules(piecesToAlign: 3)
        var board = Board(rowsNb: 3, columnsNb: 3)

        board[0, 1] = .yellow
        board[0, 0] = .red
        XCTAssertFalse(rules.isGameOver(board: board).result)
        
        board[1, 1] = .yellow
        board[0, 2] = .red
        XCTAssertFalse(rules.isGameOver(board: board).result)
        
        board[1, 2] = .yellow
        board[1, 0] = .red
        XCTAssertFalse(rules.isGameOver(board: board).result)
        
        board[2, 0] = .yellow
        board[2, 1] = .red
        XCTAssertFalse(rules.isGameOver(board: board).result)
        
        board[2, 2] = .yellow

        // Personne ne gagne, mais la partie est terminée
        let isGameOver = rules.isGameOver(board: board)
        XCTAssertTrue(isGameOver.result)
        XCTAssertEqual(isGameOver.winner, Token.empty)
    }
    func testVertical() {
        let rules = Connect4Rules(piecesToAlign: 4)
        var board = Board()

        // Alternance des joueurs dans une colonne
        board[0, 0] = Token.red
        board[0, 1] = Token.yellow
        board[1, 0] = Token.red
        board[1, 1] = Token.yellow
        board[2, 0] = Token.red
        board[2, 1] = Token.yellow

        // Rouge complète la colonne
        board[3, 0] = Token.red

        // Le rouge gagne
        let isGameOver = rules.isGameOver(board: board)
        XCTAssertTrue(isGameOver.result)
        XCTAssertEqual(isGameOver.winner, Token.red)
    }
    func testDiagonalDescending() {
        let rules = TicTacToeRules(piecesToAlign: 3)
        var board = Board(rowsNb: 3, columnsNb: 3)

        // Placer les jetons pour une victoire diagonale ascendante
        board[2, 0] = Token.red
        board[2, 1] = Token.yellow
        
        board[1, 1] = Token.red
        board[0, 0] = Token.yellow
        
        board[0, 2] = Token.red

        // Rouge gagne ici
        let isGameOver = rules.isGameOver(board: board)
        XCTAssertTrue(isGameOver.result)
        XCTAssertEqual(isGameOver.winner, Token.red)
    }

    func testDiagonalAscending() {
        let rules = TicTacToeRules(piecesToAlign: 3)
        var board = Board(rowsNb: 3, columnsNb: 3)

        // Placer les jetons pour une victoire diagonale descendante
        board[0, 2] = .yellow
        board[1, 2] = .red
        
        board[1, 1] = .yellow
        board[0, 0] = .red
        
        board[2, 0] = .yellow
        // Rouge gagne ici
        let isGameOver = rules.isGameOver(board: board)
        XCTAssertTrue(isGameOver.result)
        XCTAssertEqual(isGameOver.winner, Token.yellow)
    }
}
